@@ -82,7 +82,7 @@ static void terminal_child_exited_event(VteTerminal * vte, gint status, Term * t
 #else
 static void terminal_child_exited_event(VteTerminal * vte, Term * term);
 #endif
-static void terminal_close_button_event(VteTerminal * vte, Term * term);
+static void terminal_close_button_event(GtkButton * button, Term * term);
 static gboolean terminal_tab_button_press_event(GtkWidget * widget, GdkEventButton * event, Term * term);
 static void terminal_vte_cursor_moved_event(VteTerminal * vte, Term * term);
 static gboolean terminal_vte_button_press_event(VteTerminal * vte, GdkEventButton * event, Term * term);
@@ -458,7 +458,11 @@ static void terminal_vte_size_allocate_event(GtkWidget *widget, GtkAllocation *a
 static void terminal_close_tab_activate_event(GtkAction * action, LXTerminal * terminal)
 {
     Term * term = g_ptr_array_index(terminal->terms, gtk_notebook_get_current_page(GTK_NOTEBOOK(terminal->notebook)));
-    terminal_close_button_event(VTE_TERMINAL(term->vte), term);
+#if VTE_CHECK_VERSION (0, 38, 0)
+    terminal_child_exited_event(VTE_TERMINAL(term->vte), 0, term);
+#else
+    terminal_child_exited_event(VTE_TERMINAL(term->vte), term);
+#endif
 }
 
 /* Handler for "activate" signal on File/Close Window menu item.
@@ -471,7 +475,12 @@ static void terminal_close_window_activate_event(GtkAction * action, LXTerminal
 
     /* Play it safe and delete tabs one by one. */
     while(terminal->terms->len > 0) {
-        terminal_close_button_event(NULL, g_ptr_array_index(terminal->terms, 0));
+        Term *term = g_ptr_array_index(terminal->terms, 0);
+#if VTE_CHECK_VERSION (0, 38, 0)
+        terminal_child_exited_event(VTE_TERMINAL(term->vte), 0, term);
+#else
+        terminal_child_exited_event(VTE_TERMINAL(term->vte), term);
+#endif
     }
 }
 
@@ -760,7 +769,7 @@ static void terminal_about_activate_event(GtkAction * action, LXTerminal * termi
     gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about_dlg), VERSION);
     gtk_about_dialog_set_program_name (GTK_ABOUT_DIALOG(about_dlg), _("LXTerminal"));
     gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(about_dlg), gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/icons/hicolor/128x128/apps/lxterminal.png", NULL));
-    gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about_dlg), _("Copyright (C) 2008-2017"));
+    gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about_dlg), _("Copyright (C) 2008-2018"));
     gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about_dlg), _("Terminal emulator for LXDE project"));
     gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about_dlg), "This program is free software; you can redistribute it and/or\nmodify it under the terms of the GNU General Public License\nas published by the Free Software Foundation; either version 2\nof the License, or (at your option) any later version.\n\nThis program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License\nalong with this program; if not, write to the Free Software\nFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.");
     gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about_dlg), "http://lxde.org/");
@@ -792,7 +801,7 @@ static void terminal_switch_page_event(GtkNotebook * notebook, GtkWidget * page,
 
         /* Propagate the title to the toplevel window. */
         const gchar * title = gtk_label_get_text(GTK_LABEL(term->label));
-        gtk_window_set_title(GTK_WINDOW(terminal->window), ((title != NULL) ? title : _("LXTerminal")));
+        gtk_window_set_title(GTK_WINDOW(terminal->window), ((title != NULL) ? title : _("LXTerminal Terminal Emulator")));
 
         /* Wait for its size to be allocated, then set its geometry */
         g_signal_connect(notebook, "size-allocate", G_CALLBACK(terminal_vte_size_allocate_event), term);
@@ -872,6 +881,8 @@ static void terminal_child_exited_event(VteTerminal * vte, Term * term)
 {
     LXTerminal * terminal = term->parent;
 
+    g_signal_handler_disconnect(G_OBJECT(term->vte), term->exit_handler_id);
+
     /* Last tab being deleted.  Deallocate memory and close the window. */
     if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(terminal->notebook)) == 1)
     {
@@ -905,12 +916,12 @@ static void terminal_child_exited_event(VteTerminal * vte, Term * term)
 }
 
 /* Adapter for "activate" signal on Close button of tab and File/Close Tab menu item and accelerator. */
-static void terminal_close_button_event(VteTerminal * vte, Term * term)
+static void terminal_close_button_event(GtkButton * button, Term * term)
 {
 #if VTE_CHECK_VERSION (0, 38, 0)
-    terminal_child_exited_event(vte, 0, term);
+    terminal_child_exited_event(VTE_TERMINAL(term->vte), 0, term);
 #else
-    terminal_child_exited_event(vte, term);
+    terminal_child_exited_event(VTE_TERMINAL(term->vte), term);
 #endif
 }
 
@@ -1269,7 +1280,7 @@ static Term * terminal_new(LXTerminal * terminal, const gchar * label, const gch
     g_signal_connect(G_OBJECT(term->vte), "button-press-event", G_CALLBACK(terminal_vte_button_press_event), term);
     g_signal_connect(G_OBJECT(term->vte), "button-release-event", G_CALLBACK(terminal_vte_button_release_event), term);
     g_signal_connect(G_OBJECT(term->vte), "commit", G_CALLBACK(terminal_vte_commit), term);
-    g_signal_connect(G_OBJECT(term->vte), "child-exited", G_CALLBACK(terminal_child_exited_event), term);
+    term->exit_handler_id = g_signal_connect(G_OBJECT(term->vte), "child-exited", G_CALLBACK(terminal_child_exited_event), term);
     g_signal_connect(G_OBJECT(term->vte), "cursor-moved", G_CALLBACK(terminal_vte_cursor_moved_event), term);
     g_signal_connect(G_OBJECT(term->vte), "window-title-changed", G_CALLBACK(terminal_window_title_changed_event), term);
 
@@ -1438,6 +1449,15 @@ gboolean lxterminal_process_arguments(gint argc, gchar * * argv, CommandArgument
     }
     }
     /* Handle --loginshell. */
+    if (arguments->command != NULL && cmd_len <= 2) {
+	/* Force using login shell if it has only 1 command, and command is not
+	 * in PATH. */
+        gchar * program_path = g_find_program_in_path(arguments->command[0]);
+        if (program_path == NULL) {
+            arguments->login_shell = TRUE;
+        }
+        g_free(program_path);
+    }
     if (arguments->login_shell == TRUE)
     {
         const gchar * shell = terminal_get_preferred_shell();
@@ -1522,7 +1542,11 @@ LXTerminal * lxterminal_initialize(LXTermWindow * lxtermwin, CommandArguments *
     g_object_weak_ref(G_OBJECT(terminal->window), (GWeakNotify) terminal_window_exit, terminal);
 
     /* Create a vertical box as the child of the toplevel window. */
+#if GTK_CHECK_VERSION(3, 0, 0)
+    terminal->box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 1);
+#else
     terminal->box = gtk_vbox_new(FALSE, 1);
+#endif
     gtk_container_add(GTK_CONTAINER(terminal->window), terminal->box);
 
     /* Create the menu bar as the child of the vertical box. */
