@@ -22,6 +22,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <vte/vte.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "lxterminal.h"
 #include "setting.h"
@@ -224,37 +225,51 @@ static gboolean preferences_dialog_generic_focus_out_event(GtkWidget * widget, G
     return FALSE;
 }
 
-/* Specific "focus-out-event" handler for shortcut GtkEntry events */
-static gboolean preferences_dialog_shortcut_focus_out_event(GtkWidget * widget, GdkEventFocus * event, gchar * s)
+/* Specific "key-press-event" handler for shortcut GtkEntry events */
+static gboolean preferences_dialog_shortcut_key_press_event(GtkWidget * widget, GdkEventKey * ekey, gchar ** s)
 {
-    guint key = 0;
-    GdkModifierType mods = 0;
-    const gchar * cur = gtk_entry_get_text(GTK_ENTRY(widget));
+    guint key = ekey->keyval;
+    guint mod = ekey->state & gtk_accelerator_get_default_mod_mask();
+    gchar * lbl;
     GList * sib;
 
-    if(g_strcmp0(s, cur) == 0)
-        /* Nothing changed. */
+    /* Prevent Tab being used as one of the keys, and return false if doing so. */
+    if (key == GDK_KEY_Tab || key == GDK_KEY_ISO_Left_Tab) {
         return FALSE;
+    }
 
-    /* Look for dupplicate accelerator. */
-    for(sib = gtk_container_get_children(GTK_CONTAINER(gtk_widget_get_parent(widget))); sib; sib = sib->next)
-	if(GTK_IS_ENTRY(sib->data) && GTK_WIDGET(sib->data) != widget && !g_strcmp0(cur, gtk_entry_get_text(GTK_ENTRY(sib->data))))
-	{
-	    gtk_entry_set_text(GTK_ENTRY(widget), s);
-	    return FALSE;
+    lbl = gtk_accelerator_get_label(key, mod);
+
+    /* Look for duplicate accelerator. */
+    for(sib = gtk_container_get_children(GTK_CONTAINER(gtk_widget_get_parent(widget))); sib; sib = sib->next) {
+	if(GTK_IS_ENTRY(sib->data) && GTK_WIDGET(sib->data) != widget && !g_strcmp0(lbl, gtk_entry_get_text(GTK_ENTRY(sib->data)))) {
+            goto free_lbl;
 	}
-
-    gtk_accelerator_parse(cur, &key, &mods);
+    }
 
     /* Make sure accelerator is valid. */
-    if( ! (key == 0 && mods == 0) && gtk_accelerator_valid(key, mods))
-    {
-        g_free(s);
-        s = g_strdup(cur);
-    } else {
-        gtk_entry_set_text(GTK_ENTRY(widget), s);
+    if (!gtk_accelerator_valid(key, mod)) {
+        goto free_lbl;
     }
-    return FALSE;
+
+    g_free(*s);
+    *s = gtk_accelerator_name(key, mod);
+    gtk_entry_set_text(GTK_ENTRY(widget), lbl);
+
+free_lbl:
+    g_free(lbl);
+    return TRUE;
+}
+
+static void accel_set_label(const gchar * name, GtkWidget * w)
+{
+    guint key;
+    GdkModifierType mods;
+    gchar * label;
+    gtk_accelerator_parse(name, &key, &mods);
+    label = gtk_accelerator_get_label(key, mods);
+    gtk_entry_set_text(GTK_ENTRY(w), label);
+    g_free(label);
 }
 
 /* Initialize and display the preferences dialog. */
@@ -314,7 +329,6 @@ void terminal_preferences_dialog(GtkAction * action, LXTerminal * terminal)
     GtkComboBox *w2 = GTK_COMBO_BOX(gtk_builder_get_object(builder, "combobox_color_preset"));
     gboolean preset_is_set = FALSE;
     for (i=0; ; i++) {
-        gchar *some_data;
         gtk_list_store_insert_with_values (w3, NULL, -1, 0, color_presets[i].name, -1);
 
         if (g_strcmp0(color_presets[i].name, setting->color_preset) == 0) {
@@ -416,77 +430,28 @@ void terminal_preferences_dialog(GtkAction * action, LXTerminal * terminal)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), setting->disable_confirm);
     g_signal_connect(G_OBJECT(w), "toggled", 
         G_CALLBACK(preferences_dialog_generic_toggled_event), &setting->disable_confirm);
-    
-	/* Shortcuts */
-    w = GTK_WIDGET(gtk_builder_get_object(builder, NEW_WINDOW_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->new_window_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->new_window_accel);
 
-    w = GTK_WIDGET(gtk_builder_get_object(builder, NEW_TAB_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->new_tab_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->new_tab_accel);
+    /* Shortcuts */
+#define PREF_SETUP_SHORTCUT(OBJ, VAR) \
+    w = GTK_WIDGET(gtk_builder_get_object(builder, OBJ)); \
+    accel_set_label(VAR, w); \
+    g_signal_connect(G_OBJECT(w), "key-press-event", \
+        G_CALLBACK(preferences_dialog_shortcut_key_press_event), &VAR); \
 
-    w = GTK_WIDGET(gtk_builder_get_object(builder, CLOSE_TAB_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->close_tab_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->close_tab_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, CLOSE_WINDOW_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->close_window_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->close_window_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, COPY_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->copy_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->copy_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, PASTE_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->paste_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->paste_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, NAME_TAB_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->name_tab_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->name_tab_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, PREVIOUS_TAB_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->previous_tab_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->previous_tab_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, NEXT_TAB_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->next_tab_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->next_tab_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, MOVE_TAB_LEFT_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->move_tab_left_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->move_tab_left_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, MOVE_TAB_RIGHT_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->move_tab_right_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->move_tab_right_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, ZOOM_IN_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->zoom_in_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->zoom_in_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, ZOOM_OUT_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->zoom_out_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->zoom_out_accel);
-
-    w = GTK_WIDGET(gtk_builder_get_object(builder, ZOOM_RESET_ACCEL));
-    gtk_entry_set_text(GTK_ENTRY(w), setting->zoom_reset_accel);
-    g_signal_connect(G_OBJECT(w), "focus-out-event", 
-        G_CALLBACK(preferences_dialog_shortcut_focus_out_event), setting->zoom_reset_accel);
+    PREF_SETUP_SHORTCUT(NEW_WINDOW_ACCEL, setting->new_window_accel)
+    PREF_SETUP_SHORTCUT(NEW_TAB_ACCEL, setting->new_tab_accel)
+    PREF_SETUP_SHORTCUT(CLOSE_TAB_ACCEL, setting->close_tab_accel)
+    PREF_SETUP_SHORTCUT(CLOSE_WINDOW_ACCEL, setting->close_window_accel)
+    PREF_SETUP_SHORTCUT(COPY_ACCEL, setting->copy_accel)
+    PREF_SETUP_SHORTCUT(PASTE_ACCEL, setting->paste_accel)
+    PREF_SETUP_SHORTCUT(NAME_TAB_ACCEL, setting->name_tab_accel)
+    PREF_SETUP_SHORTCUT(PREVIOUS_TAB_ACCEL, setting->previous_tab_accel)
+    PREF_SETUP_SHORTCUT(NEXT_TAB_ACCEL, setting->next_tab_accel)
+    PREF_SETUP_SHORTCUT(MOVE_TAB_LEFT_ACCEL, setting->move_tab_left_accel)
+    PREF_SETUP_SHORTCUT(MOVE_TAB_RIGHT_ACCEL, setting->move_tab_right_accel)
+    PREF_SETUP_SHORTCUT(ZOOM_IN_ACCEL, setting->zoom_in_accel)
+    PREF_SETUP_SHORTCUT(ZOOM_OUT_ACCEL, setting->zoom_out_accel)
+    PREF_SETUP_SHORTCUT(ZOOM_RESET_ACCEL, setting->zoom_reset_accel)
 
     gtk_window_set_modal(GTK_WINDOW(GTK_DIALOG(dialog)), TRUE);
     gtk_window_set_transient_for(GTK_WINDOW(GTK_DIALOG(dialog)), 
